from typing import Tuple

from . import _LangModelType

Latin5_BulgarianCharToOrderMap: Tuple[int, ...]
win1251BulgarianCharToOrderMap: Tuple[int, ...]
BulgarianLangModel: Tuple[int, ...]
Latin5BulgarianModel: _LangModelType
Win1251BulgarianModel: _LangModelType



from typing import Any

futures: Any

class ReturnValueIgnoredError(Exception): ...

class _TracebackLogger:
    exc_info: Any
    formatted_tb: Any
    def __init__(self, exc_info) -> None: ...
    def activate(self): ...
    def clear(self): ...
    def __del__(self): ...

class Future:
    def __init__(self) -> None: ...
    def cancel(self): ...
    def cancelled(self): ...
    def running(self): ...
    def done(self): ...
    def result(self, timeout=...): ...
    def exception(self, timeout=...): ...
    def add_done_callback(self, fn): ...
    def set_result(self, result): ...
    def set_exception(self, exception): ...
    def exc_info(self): ...
    def set_exc_info(self, exc_info): ...
    def __del__(self): ...

TracebackFuture: Any
FUTURES: Any

def is_future(x): ...

class DummyExecutor:
    def submit(self, fn, *args, **kwargs): ...
    def shutdown(self, wait=...): ...

dummy_executor: Any

def run_on_executor(*args, **kwargs): ...
def return_future(f): ...
def chain_future(a, b): ...



from io import BytesIO
from typing import Any, Callable, Dict, Optional, Union

from django.contrib.sessions.backends.base import SessionBase
from django.core.handlers import base
from django.http import HttpRequest
from django.http.response import HttpResponse

_Stream = Union[BytesIO, str]
_WSGIEnviron = Dict[str, Any]

class LimitedStream:
    stream: _Stream = ...
    remaining: int = ...
    buffer: bytes = ...
    buf_size: int = ...
    def __init__(self, stream: _Stream, limit: int, buf_size: int = ...) -> None: ...
    def read(self, size: Optional[int] = ...) -> bytes: ...
    def readline(self, size: Optional[int] = ...) -> bytes: ...

class WSGIRequest(HttpRequest):
    environ: _WSGIEnviron = ...
    session: SessionBase
    encoding: Any = ...
    def __init__(self, environ: _WSGIEnviron) -> None: ...

class WSGIHandler(base.BaseHandler):
    request_class: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __call__(self, environ: _WSGIEnviron, start_response: Callable) -> HttpResponse: ...

def get_path_info(environ: _WSGIEnviron) -> str: ...
def get_script_name(environ: _WSGIEnviron) -> str: ...
def get_bytes_from_wsgi(environ: _WSGIEnviron, key: str, default: str) -> bytes: ...
def get_str_from_wsgi(environ: _WSGIEnviron, key: str, default: str) -> str: ...



from .core import (
    Argument as Argument,
    BaseCommand as BaseCommand,
    Command as Command,
    CommandCollection as CommandCollection,
    Context as Context,
    Group as Group,
    MultiCommand as MultiCommand,
    Option as Option,
    Parameter as Parameter,
)
from .decorators import (
    argument as argument,
    command as command,
    confirmation_option as confirmation_option,
    group as group,
    help_option as help_option,
    make_pass_decorator as make_pass_decorator,
    option as option,
    pass_context as pass_context,
    pass_obj as pass_obj,
    password_option as password_option,
    version_option as version_option,
)
from .exceptions import (
    Abort as Abort,
    BadArgumentUsage as BadArgumentUsage,
    BadOptionUsage as BadOptionUsage,
    BadParameter as BadParameter,
    ClickException as ClickException,
    FileError as FileError,
    MissingParameter as MissingParameter,
    NoSuchOption as NoSuchOption,
    UsageError as UsageError,
)
from .formatting import HelpFormatter as HelpFormatter, wrap_text as wrap_text
from .globals import get_current_context as get_current_context
from .parser import OptionParser as OptionParser
from .termui import (
    clear as clear,
    confirm as confirm,
    echo_via_pager as echo_via_pager,
    edit as edit,
    get_terminal_size as get_terminal_size,
    getchar as getchar,
    launch as launch,
    pause as pause,
    progressbar as progressbar,
    prompt as prompt,
    secho as secho,
    style as style,
    unstyle as unstyle,
)
from .types import (
    BOOL as BOOL,
    FLOAT as FLOAT,
    INT as INT,
    STRING as STRING,
    UNPROCESSED as UNPROCESSED,
    UUID as UUID,
    Choice as Choice,
    DateTime as DateTime,
    File as File,
    FloatRange as FloatRange,
    IntRange as IntRange,
    ParamType as ParamType,
    Path as Path,
    Tuple as Tuple,
)
from .utils import (
    echo as echo,
    format_filename as format_filename,
    get_app_dir as get_app_dir,
    get_binary_stream as get_binary_stream,
    get_os_args as get_os_args,
    get_text_stream as get_text_stream,
    open_file as open_file,
)

# Controls if click should emit the warning about the use of unicode
# literals.
disable_unicode_literals_warning: bool

__version__: str



from typing import Any, Callable, Container, Dict, Iterable, List, Optional, Pattern, Text, Union

ALLOWED_TAGS: List[Text]
ALLOWED_ATTRIBUTES: Dict[Text, List[Text]]
ALLOWED_STYLES: List[Text]
ALLOWED_PROTOCOLS: List[Text]

INVISIBLE_CHARACTERS: Text
INVISIBLE_CHARACTERS_RE: Pattern[Text]
INVISIBLE_REPLACEMENT_CHAR: Text

# A html5lib Filter class
_Filter = Any

class Cleaner(object):
    def __init__(
        self,
        tags: Container[Text] = ...,
        attributes: Any = ...,
        styles: Container[Text] = ...,
        protocols: Container[Text] = ...,
        strip: bool = ...,
        strip_comments: bool = ...,
        filters: Optional[Iterable[_Filter]] = ...,
    ) -> None: ...
    def clean(self, text: Text) -> Text: ...

_AttributeFilter = Callable[[Text, Text, Text], bool]
_AttributeDict = Dict[Text, Union[Container[Text], _AttributeFilter]]

def attribute_filter_factory(attributes: Union[_AttributeFilter, _AttributeDict, Container[Text]]) -> _AttributeFilter: ...

class BleachSanitizerFilter(object):  # TODO: derives from html5lib.sanitizer.Filter
    def __getattr__(self, item: str) -> Any: ...  # incomplete



from typing import Any

from django.utils.functional import LazyObject

# explicit dependency on standard settings to make it loaded
from . import global_settings

ENVIRONMENT_VARIABLE: str = ...
DEFAULT_CONTENT_TYPE_DEPRECATED_MSG: str = ...
FILE_CHARSET_DEPRECATED_MSG: str = ...

# required for plugin to be able to distinguish this specific instance of LazySettings from others
class _DjangoConfLazyObject(LazyObject):
    def __getattr__(self, item: Any) -> Any: ...

class LazySettings(_DjangoConfLazyObject):
    configured: bool
    def configure(self, default_settings: Any = ..., **options: Any) -> Any: ...

settings: LazySettings = ...

class Settings:
    def __init__(self, settings_module: str): ...
    def is_overridden(self, setting: str) -> bool: ...

class UserSettingsHolder: ...

class SettingsReference(str):
    def __init__(self, value: str, setting_name: str) -> None: ...



from typing import Any, Dict, List, Optional, Union
from . import engines as engines  # noqa: F401

from django.http.request import HttpRequest
from django.template.exceptions import TemplateDoesNotExist as TemplateDoesNotExist  # noqa: F401

def get_template(template_name: str, using: Optional[str] = ...) -> Any: ...
def select_template(template_name_list: Union[List[str], str], using: Optional[str] = ...) -> Any: ...
def render_to_string(
    template_name: Union[List[str], str],
    context: Optional[Dict[str, Any]] = ...,
    request: Optional[HttpRequest] = ...,
    using: Optional[str] = ...,
) -> str: ...



from typing import List, Optional

from django.db.migrations.operations.base import Operation

class MigrationOptimizer:
    def optimize(self, operations: List[Operation], app_label: Optional[str] = ...) -> List[Operation]: ...
    def optimize_inner(self, operations: List[Operation], app_label: Optional[str] = ...) -> List[Operation]: ...



import sys
from typing import List, Optional, Tuple

if sys.platform == "win32":

    _SequenceType = List[Tuple[str, Optional[str], int]]

    AdminExecuteSequence: _SequenceType
    AdminUISequence: _SequenceType
    AdvtExecuteSequence: _SequenceType
    InstallExecuteSequence: _SequenceType
    InstallUISequence: _SequenceType

    tables: List[str]



from typing import Any, Dict, Iterable, Iterator, List, Optional, Text, Tuple, TypeVar, Union

from .connections import Connection

_Gen = Union[Tuple[Any, ...], Dict[Text, Any]]
_QueryArgs = Union[Tuple[Any, ...], List[Any], Dict[Text, Any], None]
_SelfT = TypeVar("_SelfT")

class Cursor:
    connection: Connection
    description: Tuple[Text, ...]
    rownumber: int
    rowcount: int
    arraysize: int
    messages: Any
    errorhandler: Any
    lastrowid: int
    def __init__(self, connection: Connection) -> None: ...
    def __del__(self) -> None: ...
    def close(self) -> None: ...
    def setinputsizes(self, *args) -> None: ...
    def setoutputsizes(self, *args) -> None: ...
    def nextset(self) -> Optional[bool]: ...
    def mogrify(self, query: Text, args: _QueryArgs = ...) -> str: ...
    def execute(self, query: Text, args: _QueryArgs = ...) -> int: ...
    def executemany(self, query: Text, args: Iterable[_QueryArgs]) -> Optional[int]: ...
    def callproc(self, procname: Text, args: Iterable[Any] = ...) -> Any: ...
    def fetchone(self) -> Optional[_Gen]: ...
    def fetchmany(self, size: Optional[int] = ...) -> Union[Optional[_Gen], List[_Gen]]: ...
    def fetchall(self) -> Optional[Tuple[_Gen, ...]]: ...
    def scroll(self, value: int, mode: Text = ...) -> None: ...
    def __iter__(self) -> Iterator[_Gen]: ...
    def __enter__(self: _SelfT) -> _SelfT: ...
    def __exit__(self, *exc_info: Any) -> None: ...

class DictCursor(Cursor):
    def fetchone(self) -> Optional[Dict[Text, Any]]: ...
    def fetchmany(self, size: Optional[int] = ...) -> Optional[Tuple[Dict[Text, Any], ...]]: ...
    def fetchall(self) -> Optional[Tuple[Dict[Text, Any], ...]]: ...

class DictCursorMixin:
    dict_type: Any

class SSCursor(Cursor):
    # fetchall return type is incompatible with the supertype.
    def fetchall(self) -> List[_Gen]: ...  # type: ignore
    def fetchall_unbuffered(self) -> Iterator[_Gen]: ...
    def __iter__(self) -> Iterator[_Gen]: ...
    def fetchmany(self, size: Optional[int] = ...) -> List[_Gen]: ...
    def scroll(self, value: int, mode: Text = ...) -> None: ...

class SSDictCursor(DictCursorMixin, SSCursor): ...



import threading
import types
from pathlib import Path
from typing import Any, Callable, List, Optional, Set, Dict, Union, Iterator, Tuple, Iterable

from django.apps.registry import Apps

USE_INOTIFY: bool
fd: Any
RUN_RELOADER: bool
FILE_MODIFIED: int
I18N_MODIFIED: int

def gen_filenames(only_new: bool = ...) -> List[str]: ...
def clean_files(filelist: List[Optional[str]]) -> List[str]: ...
def reset_translations() -> None: ...
def inotify_code_changed(): ...
def code_changed(): ...
def check_errors(fn: Callable) -> Callable: ...
def raise_last_exception() -> None: ...
def ensure_echo_on() -> None: ...
def reloader_thread() -> None: ...
def restart_with_reloader() -> int: ...
def python_reloader(main_func: Any, args: Any, kwargs: Any) -> None: ...
def main(main_func: Any, args: Optional[Any] = ..., kwargs: Optional[Any] = ...) -> None: ...
def iter_all_python_module_files() -> Set[Path]: ...
def iter_modules_and_files(
    modules: Iterable[types.ModuleType], extra_files: Iterable[Union[str, Path]]
) -> Set[Path]: ...
def common_roots(paths: Iterable[Path]) -> Iterator[Path]: ...
def sys_path_directories() -> Iterator[Path]: ...

class BaseReloader:
    extra_files: Set[Path]
    directory_globs: Dict[Path, Set[str]]
    def __init__(self) -> None: ...
    def watch_dir(self, path: Union[str, Path], glob: str) -> None: ...
    def watch_file(self, path: Union[str, Path]) -> None: ...
    def watched_files(self, include_globs: bool = ...) -> Iterator[Path]: ...
    def wait_for_apps_ready(self, app_reg: Apps, django_main_thread: threading.Thread) -> bool: ...
    def run(self, django_main_thread: threading.Thread) -> None: ...
    def run_loop(self) -> None: ...
    def tick(self) -> Iterator[None]: ...
    @classmethod
    def check_availability(cls) -> bool: ...
    def notify_file_changed(self, path: Union[str, Path]) -> None: ...
    @property
    def should_stop(self) -> bool: ...
    def stop(self) -> None: ...

class StatReloader(BaseReloader):
    SLEEP_TIME: int = ...
    def snapshot_files(self) -> Iterator[Tuple[Path, int]]: ...

class WatchmanUnavailable(RuntimeError): ...

class WatchmanReloader(BaseReloader):
    @property
    def client(self) -> Any: ...
    def watched_roots(self, watched_files: Iterable[Path]) -> Set[Path]: ...
    def update_watches(self) -> None: ...
    def request_processed(self, **kwargs: Any) -> None: ...
    def check_server_status(self, inner_ex: Optional[BaseException] = ...) -> bool: ...

def get_reloader() -> BaseReloader: ...
def start_django(reloader: BaseReloader, main_func: Callable, *args: Any, **kwargs: Any) -> None: ...
def run_with_reloader(main_func: Callable, *args: Any, **kwargs: Any) -> None: ...



import sys
from typing import Any, AnyStr, Callable, Dict, Generic, Iterable, List, Optional, Sequence, Text, Tuple, Union

if sys.version_info >= (3, 6):
    from os import PathLike

if sys.version_info >= (3, 9):
    from types import GenericAlias

DEFAULT_IGNORES: List[str]

if sys.version_info >= (3, 6):
    def cmp(
        f1: Union[bytes, Text, PathLike[AnyStr]], f2: Union[bytes, Text, PathLike[AnyStr]], shallow: Union[int, bool] = ...
    ) -> bool: ...
    def cmpfiles(
        a: Union[AnyStr, PathLike[AnyStr]],
        b: Union[AnyStr, PathLike[AnyStr]],
        common: Iterable[AnyStr],
        shallow: Union[int, bool] = ...,
    ) -> Tuple[List[AnyStr], List[AnyStr], List[AnyStr]]: ...

else:
    def cmp(f1: Union[bytes, Text], f2: Union[bytes, Text], shallow: Union[int, bool] = ...) -> bool: ...
    def cmpfiles(
        a: AnyStr, b: AnyStr, common: Iterable[AnyStr], shallow: Union[int, bool] = ...
    ) -> Tuple[List[AnyStr], List[AnyStr], List[AnyStr]]: ...

class dircmp(Generic[AnyStr]):
    if sys.version_info >= (3, 6):
        def __init__(
            self,
            a: Union[AnyStr, PathLike[AnyStr]],
            b: Union[AnyStr, PathLike[AnyStr]],
            ignore: Optional[Sequence[AnyStr]] = ...,
            hide: Optional[Sequence[AnyStr]] = ...,
        ) -> None: ...
    else:
        def __init__(
            self, a: AnyStr, b: AnyStr, ignore: Optional[Sequence[AnyStr]] = ..., hide: Optional[Sequence[AnyStr]] = ...
        ) -> None: ...
    left: AnyStr
    right: AnyStr
    hide: Sequence[AnyStr]
    ignore: Sequence[AnyStr]
    # These properties are created at runtime by __getattr__
    subdirs: Dict[AnyStr, dircmp[AnyStr]]
    same_files: List[AnyStr]
    diff_files: List[AnyStr]
    funny_files: List[AnyStr]
    common_dirs: List[AnyStr]
    common_files: List[AnyStr]
    common_funny: List[AnyStr]
    common: List[AnyStr]
    left_only: List[AnyStr]
    right_only: List[AnyStr]
    left_list: List[AnyStr]
    right_list: List[AnyStr]
    def report(self) -> None: ...
    def report_partial_closure(self) -> None: ...
    def report_full_closure(self) -> None: ...
    methodmap: Dict[str, Callable[[], None]]
    def phase0(self) -> None: ...
    def phase1(self) -> None: ...
    def phase2(self) -> None: ...
    def phase3(self) -> None: ...
    def phase4(self) -> None: ...
    def phase4_closure(self) -> None: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

if sys.version_info >= (3,):
    def clear_cache() -> None: ...



from typing import Callable, Optional, TypeVar, overload

from . import _ConverterType

_T = TypeVar("_T")

def optional(converter: _ConverterType[_T]) -> _ConverterType[Optional[_T]]: ...
@overload
def default_if_none(default: _T) -> _ConverterType[_T]: ...
@overload
def default_if_none(*, factory: Callable[[], _T]) -> _ConverterType[_T]: ...



from datetime import timedelta
from types import TracebackType
from typing import (
    Any,
    ByteString,
    Callable,
    ContextManager,
    Dict,
    Iterable,
    List,
    NoReturn,
    Optional,
    Text,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from .blueprints import Blueprint
from .config import Config
from .ctx import AppContext, RequestContext
from .helpers import _PackageBoundObject
from .testing import FlaskClient
from .wrappers import Response

def setupmethod(f: Any): ...

_T = TypeVar("_T")

_ExcInfo = Tuple[Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]]
_StartResponse = Callable[[str, List[Tuple[str, str]], Optional[_ExcInfo]], Callable[[bytes], Any]]
_WSGICallable = Callable[[Dict[Text, Any], _StartResponse], Iterable[bytes]]

_Status = Union[str, int]
_Headers = Union[Dict[Any, Any], List[Tuple[Any, Any]]]
_Body = Union[Text, ByteString, Dict[Text, Any], Response, _WSGICallable]
_ViewFuncReturnType = Union[_Body, Tuple[_Body, _Status, _Headers], Tuple[_Body, _Status], Tuple[_Body, _Headers]]

_ViewFunc = Union[Callable[..., NoReturn], Callable[..., _ViewFuncReturnType]]
_VT = TypeVar("_VT", bound=_ViewFunc)

class Flask(_PackageBoundObject):
    request_class: type = ...
    response_class: type = ...
    jinja_environment: type = ...
    app_ctx_globals_class: type = ...
    config_class: Type[Config] = ...
    testing: Any = ...
    secret_key: Union[Text, bytes, None] = ...
    session_cookie_name: Any = ...
    permanent_session_lifetime: timedelta = ...
    send_file_max_age_default: timedelta = ...
    use_x_sendfile: Any = ...
    json_encoder: Any = ...
    json_decoder: Any = ...
    jinja_options: Any = ...
    default_config: Any = ...
    url_rule_class: type = ...
    test_client_class: type = ...
    test_cli_runner_class: type = ...
    session_interface: Any = ...
    import_name: str = ...
    template_folder: str = ...
    root_path: Union[str, Text] = ...
    static_url_path: Any = ...
    static_folder: Optional[str] = ...
    instance_path: Union[str, Text] = ...
    config: Config = ...
    view_functions: Any = ...
    error_handler_spec: Any = ...
    url_build_error_handlers: Any = ...
    before_request_funcs: Dict[Optional[str], List[Callable[[], Any]]] = ...
    before_first_request_funcs: List[Callable[[], None]] = ...
    after_request_funcs: Dict[Optional[str], List[Callable[[Response], Response]]] = ...
    teardown_request_funcs: Dict[Optional[str], List[Callable[[Optional[Exception]], Any]]] = ...
    teardown_appcontext_funcs: List[Callable[[Optional[Exception]], Any]] = ...
    url_value_preprocessors: Any = ...
    url_default_functions: Any = ...
    template_context_processors: Any = ...
    shell_context_processors: Any = ...
    blueprints: Any = ...
    extensions: Any = ...
    url_map: Any = ...
    subdomain_matching: Any = ...
    cli: Any = ...
    def __init__(
        self,
        import_name: str,
        static_url_path: Optional[str] = ...,
        static_folder: Optional[str] = ...,
        static_host: Optional[str] = ...,
        host_matching: bool = ...,
        subdomain_matching: bool = ...,
        template_folder: str = ...,
        instance_path: Optional[str] = ...,
        instance_relative_config: bool = ...,
        root_path: Optional[str] = ...,
    ) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def propagate_exceptions(self) -> bool: ...
    @property
    def preserve_context_on_exception(self): ...
    @property
    def logger(self): ...
    @property
    def jinja_env(self): ...
    @property
    def got_first_request(self) -> bool: ...
    def make_config(self, instance_relative: bool = ...): ...
    def auto_find_instance_path(self): ...
    def open_instance_resource(self, resource: Union[str, Text], mode: str = ...): ...
    templates_auto_reload: Any = ...
    def create_jinja_environment(self): ...
    def create_global_jinja_loader(self): ...
    def select_jinja_autoescape(self, filename: Any): ...
    def update_template_context(self, context: Any) -> None: ...
    def make_shell_context(self): ...
    env: Optional[str] = ...
    debug: bool = ...
    def run(
        self,
        host: Optional[str] = ...,
        port: Optional[Union[int, str]] = ...,
        debug: Optional[bool] = ...,
        load_dotenv: bool = ...,
        **options: Any,
    ) -> None: ...
    def test_client(self, use_cookies: bool = ..., **kwargs: Any) -> FlaskClient[Response]: ...
    def test_cli_runner(self, **kwargs: Any): ...
    def open_session(self, request: Any): ...
    def save_session(self, session: Any, response: Any): ...
    def make_null_session(self): ...
    def register_blueprint(self, blueprint: Blueprint, **options: Any) -> None: ...
    def iter_blueprints(self): ...
    def add_url_rule(
        self,
        rule: str,
        endpoint: Optional[str] = ...,
        view_func: _ViewFunc = ...,
        provide_automatic_options: Optional[bool] = ...,
        **options: Any,
    ) -> None: ...
    def route(self, rule: str, **options: Any) -> Callable[[_VT], _VT]: ...
    def endpoint(self, endpoint: str) -> Callable[[Callable[..., _T]], Callable[..., _T]]: ...
    def errorhandler(
        self, code_or_exception: Union[int, Type[Exception]]
    ) -> Callable[[Callable[..., _T]], Callable[..., _T]]: ...
    def register_error_handler(self, code_or_exception: Union[int, Type[Exception]], f: Callable[..., Any]) -> None: ...
    def template_filter(self, name: Optional[Any] = ...): ...
    def add_template_filter(self, f: Any, name: Optional[Any] = ...) -> None: ...
    def template_test(self, name: Optional[Any] = ...): ...
    def add_template_test(self, f: Any, name: Optional[Any] = ...) -> None: ...
    def template_global(self, name: Optional[Any] = ...): ...
    def add_template_global(self, f: Any, name: Optional[Any] = ...) -> None: ...
    def before_request(self, f: Callable[[], _T]) -> Callable[[], _T]: ...
    def before_first_request(self, f: Callable[[], _T]) -> Callable[[], _T]: ...
    def after_request(self, f: Callable[[Response], Response]) -> Callable[[Response], Response]: ...
    def teardown_request(self, f: Callable[[Optional[Exception]], _T]) -> Callable[[Optional[Exception]], _T]: ...
    def teardown_appcontext(self, f: Callable[[Optional[Exception]], _T]) -> Callable[[Optional[Exception]], _T]: ...
    def context_processor(self, f: Any): ...
    def shell_context_processor(self, f: Any): ...
    def url_value_preprocessor(self, f: Any): ...
    def url_defaults(self, f: Any): ...
    def handle_http_exception(self, e: Any): ...
    def trap_http_exception(self, e: Any): ...
    def handle_user_exception(self, e: Any): ...
    def handle_exception(self, e: Any): ...
    def log_exception(self, exc_info: Any) -> None: ...
    def raise_routing_exception(self, request: Any) -> None: ...
    def dispatch_request(self): ...
    def full_dispatch_request(self): ...
    def finalize_request(self, rv: Any, from_error_handler: bool = ...): ...
    def try_trigger_before_first_request_functions(self): ...
    def make_default_options_response(self): ...
    def should_ignore_error(self, error: Any): ...
    def make_response(self, rv: Any): ...
    def create_url_adapter(self, request: Any): ...
    def inject_url_defaults(self, endpoint: Any, values: Any) -> None: ...
    def handle_url_build_error(self, error: Any, endpoint: Any, values: Any): ...
    def preprocess_request(self): ...
    def process_response(self, response: Any): ...
    def do_teardown_request(self, exc: Any = ...) -> None: ...
    def do_teardown_appcontext(self, exc: Any = ...) -> None: ...
    def app_context(self) -> AppContext: ...
    def request_context(self, environ: Any): ...
    def test_request_context(self, *args: Any, **kwargs: Any) -> ContextManager[RequestContext]: ...
    def wsgi_app(self, environ: Any, start_response: Any): ...
    def __call__(self, environ: Any, start_response: Any): ...
    # These are not preset at runtime but we add them since monkeypatching this
    # class is quite common.
    def __setattr__(self, name: str, value: Any): ...
    def __getattr__(self, name: str): ...



from typing import Any

from tornado.util import Configurable

ssl: Any
certifi: Any
xrange: Any
ssl_match_hostname: Any
SSLCertificateError: Any

def bind_sockets(port, address=..., family=..., backlog=..., flags=...): ...
def bind_unix_socket(file, mode=..., backlog=...): ...
def add_accept_handler(sock, callback, io_loop=...): ...
def is_valid_ip(ip): ...

class Resolver(Configurable):
    @classmethod
    def configurable_base(cls): ...
    @classmethod
    def configurable_default(cls): ...
    def resolve(self, host, port, family=..., callback=...): ...
    def close(self): ...

class ExecutorResolver(Resolver):
    io_loop: Any
    executor: Any
    close_executor: Any
    def initialize(self, io_loop=..., executor=..., close_executor=...): ...
    def close(self): ...
    def resolve(self, host, port, family=...): ...

class BlockingResolver(ExecutorResolver):
    def initialize(self, io_loop=...): ...

class ThreadedResolver(ExecutorResolver):
    def initialize(self, io_loop=..., num_threads=...): ...

class OverrideResolver(Resolver):
    resolver: Any
    mapping: Any
    def initialize(self, resolver, mapping): ...
    def close(self): ...
    def resolve(self, host, port, *args, **kwargs): ...

def ssl_options_to_context(ssl_options): ...
def ssl_wrap_socket(socket, ssl_options, server_hostname=..., **kwargs): ...



import sys
from _typeshed.wsgi import WSGIEnvironment
from datetime import datetime, timedelta
from typing import (
    Any,
    Callable,
    Dict,
    Iterable,
    List,
    Mapping,
    Optional,
    SupportsInt,
    Text,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)

from .datastructures import (
    Accept,
    Authorization,
    ContentRange,
    ETags,
    Headers,
    HeaderSet,
    IfRange,
    Range,
    RequestCacheControl,
    TypeConversionDict,
    WWWAuthenticate,
)

if sys.version_info < (3,):
    _Str = TypeVar("_Str", str, unicode)
    _ToBytes = Union[bytes, bytearray, buffer, unicode]
    _ETagData = Union[str, unicode, bytearray, buffer, memoryview]
else:
    _Str = str
    _ToBytes = Union[bytes, bytearray, memoryview, str]
    _ETagData = Union[bytes, bytearray, memoryview]

_T = TypeVar("_T")
_U = TypeVar("_U")

HTTP_STATUS_CODES: Dict[int, str]

def wsgi_to_bytes(data: Union[bytes, Text]) -> bytes: ...
def bytes_to_wsgi(data: bytes) -> str: ...
def quote_header_value(value: Any, extra_chars: str = ..., allow_token: bool = ...) -> str: ...
def unquote_header_value(value: _Str, is_filename: bool = ...) -> _Str: ...
def dump_options_header(header: Optional[_Str], options: Mapping[_Str, Any]) -> _Str: ...
def dump_header(iterable: Union[Iterable[Any], Dict[_Str, Any]], allow_token: bool = ...) -> _Str: ...
def parse_list_header(value: _Str) -> List[_Str]: ...
@overload
def parse_dict_header(value: Union[bytes, Text]) -> Dict[Text, Optional[Text]]: ...
@overload
def parse_dict_header(value: Union[bytes, Text], cls: Type[_T]) -> _T: ...
@overload
def parse_options_header(value: None, multiple: bool = ...) -> Tuple[str, Dict[str, Optional[str]]]: ...
@overload
def parse_options_header(value: _Str) -> Tuple[_Str, Dict[_Str, Optional[_Str]]]: ...

# actually returns Tuple[_Str, Dict[_Str, Optional[_Str]], ...]
@overload
def parse_options_header(value: _Str, multiple: bool = ...) -> Tuple[Any, ...]: ...
@overload
def parse_accept_header(value: Optional[Text]) -> Accept: ...
@overload
def parse_accept_header(value: Optional[_Str], cls: Callable[[Optional[List[Tuple[str, float]]]], _T]) -> _T: ...
@overload
def parse_cache_control_header(
    value: Union[None, bytes, Text], on_update: Optional[Callable[[RequestCacheControl], Any]] = ...
) -> RequestCacheControl: ...
@overload
def parse_cache_control_header(
    value: Union[None, bytes, Text], on_update: _T, cls: Callable[[Dict[Text, Optional[Text]], _T], _U]
) -> _U: ...
@overload
def parse_cache_control_header(
    value: Union[None, bytes, Text], *, cls: Callable[[Dict[Text, Optional[Text]], None], _U]
) -> _U: ...
def parse_set_header(value: Text, on_update: Optional[Callable[[HeaderSet], Any]] = ...) -> HeaderSet: ...
def parse_authorization_header(value: Union[None, bytes, Text]) -> Optional[Authorization]: ...
def parse_www_authenticate_header(
    value: Union[None, bytes, Text], on_update: Optional[Callable[[WWWAuthenticate], Any]] = ...
) -> WWWAuthenticate: ...
def parse_if_range_header(value: Optional[Text]) -> IfRange: ...
def parse_range_header(value: Optional[Text], make_inclusive: bool = ...) -> Optional[Range]: ...
def parse_content_range_header(
    value: Optional[Text], on_update: Optional[Callable[[ContentRange], Any]] = ...
) -> Optional[ContentRange]: ...
def quote_etag(etag: _Str, weak: bool = ...) -> _Str: ...
def unquote_etag(etag: Optional[_Str]) -> Tuple[Optional[_Str], Optional[_Str]]: ...
def parse_etags(value: Optional[Text]) -> ETags: ...
def generate_etag(data: _ETagData) -> str: ...
def parse_date(value: Optional[str]) -> Optional[datetime]: ...
def cookie_date(expires: Union[None, float, datetime] = ...) -> str: ...
def http_date(timestamp: Union[None, float, datetime] = ...) -> str: ...
def parse_age(value: Optional[SupportsInt] = ...) -> Optional[timedelta]: ...
def dump_age(age: Union[None, timedelta, SupportsInt]) -> Optional[str]: ...
def is_resource_modified(
    environ: WSGIEnvironment,
    etag: Optional[Text] = ...,
    data: Optional[_ETagData] = ...,
    last_modified: Union[None, Text, datetime] = ...,
    ignore_if_range: bool = ...,
) -> bool: ...
def remove_entity_headers(headers: Union[List[Tuple[Text, Text]], Headers], allowed: Iterable[Text] = ...) -> None: ...
def remove_hop_by_hop_headers(headers: Union[List[Tuple[Text, Text]], Headers]) -> None: ...
def is_entity_header(header: Text) -> bool: ...
def is_hop_by_hop_header(header: Text) -> bool: ...
@overload
def parse_cookie(
    header: Union[None, WSGIEnvironment, Text, bytes], charset: Text = ..., errors: Text = ...
) -> TypeConversionDict[Any, Any]: ...
@overload
def parse_cookie(
    header: Union[None, WSGIEnvironment, Text, bytes],
    charset: Text = ...,
    errors: Text = ...,
    cls: Optional[Callable[[Iterable[Tuple[Text, Text]]], _T]] = ...,
) -> _T: ...
def dump_cookie(
    key: _ToBytes,
    value: _ToBytes = ...,
    max_age: Union[None, float, timedelta] = ...,
    expires: Union[None, Text, float, datetime] = ...,
    path: Union[None, Tuple[Any, ...], str, bytes] = ...,
    domain: Union[None, str, bytes] = ...,
    secure: bool = ...,
    httponly: bool = ...,
    charset: Text = ...,
    sync_expires: bool = ...,
) -> str: ...
def is_byte_range_valid(start: Optional[int], stop: Optional[int], length: Optional[int]) -> bool: ...



import sys
from pathlib import Path
from typing import BinaryIO, Callable, Optional, Union

_Path = Union[str, Path, BinaryIO]

class ZipAppError(ValueError): ...

if sys.version_info >= (3, 7):
    def create_archive(
        source: _Path,
        target: Optional[_Path] = ...,
        interpreter: Optional[str] = ...,
        main: Optional[str] = ...,
        filter: Optional[Callable[[Path], bool]] = ...,
        compressed: bool = ...,
    ) -> None: ...

else:
    def create_archive(
        source: _Path, target: Optional[_Path] = ..., interpreter: Optional[str] = ..., main: Optional[str] = ...
    ) -> None: ...

def get_interpreter(archive: _Path) -> str: ...



from typing import Any

from .packages.urllib3.exceptions import HTTPError as BaseHTTPError

class RequestException(IOError):
    response: Any
    request: Any
    def __init__(self, *args, **kwargs) -> None: ...

class HTTPError(RequestException): ...
class ConnectionError(RequestException): ...
class ProxyError(ConnectionError): ...
class SSLError(ConnectionError): ...
class Timeout(RequestException): ...
class ConnectTimeout(ConnectionError, Timeout): ...
class ReadTimeout(Timeout): ...
class URLRequired(RequestException): ...
class TooManyRedirects(RequestException): ...
class MissingSchema(RequestException, ValueError): ...
class InvalidSchema(RequestException, ValueError): ...
class InvalidURL(RequestException, ValueError): ...
class InvalidHeader(RequestException, ValueError): ...
class InvalidProxyURL(InvalidURL): ...
class ChunkedEncodingError(RequestException): ...
class ContentDecodingError(RequestException, BaseHTTPError): ...
class StreamConsumedError(RequestException, TypeError): ...
class RetryError(RequestException): ...
class UnrewindableBodyError(RequestException): ...
class RequestsWarning(Warning): ...
class FileModeWarning(RequestsWarning, DeprecationWarning): ...
class RequestsDependencyWarning(RequestsWarning): ...



from typing import Optional, Sequence, Tuple, Union

from django.db.models.constraints import BaseConstraint
from django.db.models.expressions import Combinable
from django.db.models.query_utils import Q

class ExclusionConstraint(BaseConstraint):
    expressions: Sequence[Tuple[Union[str, Combinable], str]]
    index_type: str
    condition: Optional[Q]
    def __init__(
        self,
        *,
        name: str,
        expressions: Sequence[Tuple[Union[str, Combinable], str]],
        condition: Optional[Q] = ...,
        index_type: Optional[str] = ...,
    ): ...



import sys
from typing import Any, AsyncIterator, Awaitable, Callable, Iterable, Optional, Tuple, Union

from . import events, protocols, transports

_ClientConnectedCallback = Callable[[StreamReader, StreamWriter], Optional[Awaitable[None]]]

if sys.version_info < (3, 8):
    class IncompleteReadError(EOFError):
        expected: Optional[int]
        partial: bytes
        def __init__(self, partial: bytes, expected: Optional[int]) -> None: ...
    class LimitOverrunError(Exception):
        consumed: int
        def __init__(self, message: str, consumed: int) -> None: ...

async def open_connection(
    host: Optional[str] = ...,
    port: Optional[Union[int, str]] = ...,
    *,
    loop: Optional[events.AbstractEventLoop] = ...,
    limit: int = ...,
    ssl_handshake_timeout: Optional[float] = ...,
    **kwds: Any,
) -> Tuple[StreamReader, StreamWriter]: ...
async def start_server(
    client_connected_cb: _ClientConnectedCallback,
    host: Optional[str] = ...,
    port: Optional[Union[int, str]] = ...,
    *,
    loop: Optional[events.AbstractEventLoop] = ...,
    limit: int = ...,
    ssl_handshake_timeout: Optional[float] = ...,
    **kwds: Any,
) -> events.AbstractServer: ...

if sys.platform != "win32":
    if sys.version_info >= (3, 7):
        from os import PathLike

        _PathType = Union[str, PathLike[str]]
    else:
        _PathType = str
    async def open_unix_connection(
        path: Optional[_PathType] = ..., *, loop: Optional[events.AbstractEventLoop] = ..., limit: int = ..., **kwds: Any
    ) -> Tuple[StreamReader, StreamWriter]: ...
    async def start_unix_server(
        client_connected_cb: _ClientConnectedCallback,
        path: Optional[_PathType] = ...,
        *,
        loop: Optional[events.AbstractEventLoop] = ...,
        limit: int = ...,
        **kwds: Any,
    ) -> events.AbstractServer: ...

class FlowControlMixin(protocols.Protocol): ...

class StreamReaderProtocol(FlowControlMixin, protocols.Protocol):
    def __init__(
        self,
        stream_reader: StreamReader,
        client_connected_cb: Optional[_ClientConnectedCallback] = ...,
        loop: Optional[events.AbstractEventLoop] = ...,
    ) -> None: ...
    def connection_made(self, transport: transports.BaseTransport) -> None: ...
    def connection_lost(self, exc: Optional[Exception]) -> None: ...
    def data_received(self, data: bytes) -> None: ...
    def eof_received(self) -> bool: ...

class StreamWriter:
    def __init__(
        self,
        transport: transports.BaseTransport,
        protocol: protocols.BaseProtocol,
        reader: Optional[StreamReader],
        loop: events.AbstractEventLoop,
    ) -> None: ...
    @property
    def transport(self) -> transports.BaseTransport: ...
    def write(self, data: bytes) -> None: ...
    def writelines(self, data: Iterable[bytes]) -> None: ...
    def write_eof(self) -> None: ...
    def can_write_eof(self) -> bool: ...
    def close(self) -> None: ...
    if sys.version_info >= (3, 7):
        def is_closing(self) -> bool: ...
        async def wait_closed(self) -> None: ...
    def get_extra_info(self, name: str, default: Any = ...) -> Any: ...
    async def drain(self) -> None: ...

class StreamReader:
    def __init__(self, limit: int = ..., loop: Optional[events.AbstractEventLoop] = ...) -> None: ...
    def exception(self) -> Exception: ...
    def set_exception(self, exc: Exception) -> None: ...
    def set_transport(self, transport: transports.BaseTransport) -> None: ...
    def feed_eof(self) -> None: ...
    def at_eof(self) -> bool: ...
    def feed_data(self, data: bytes) -> None: ...
    async def readline(self) -> bytes: ...
    async def readuntil(self, separator: bytes = ...) -> bytes: ...
    async def read(self, n: int = ...) -> bytes: ...
    async def readexactly(self, n: int) -> bytes: ...
    def __aiter__(self) -> AsyncIterator[bytes]: ...
    async def __anext__(self) -> bytes: ...



from typing import Any, Dict, Mapping, Optional, Sequence, Sized

from django.forms import Form

TOTAL_FORM_COUNT: str = ...
INITIAL_FORM_COUNT: str = ...
MIN_NUM_FORM_COUNT: str = ...
MAX_NUM_FORM_COUNT: str = ...
ORDERING_FIELD_NAME: str = ...
DELETION_FIELD_NAME: str = ...

DEFAULT_MIN_NUM: int = ...
DEFAULT_MAX_NUM: int = ...

class ManagementForm(Form):
    cleaned_data: Dict[str, Optional[int]]

class BaseFormSet(Sized, Mapping[str, Any]):
    is_bound: Any = ...
    prefix: Any = ...
    auto_id: Any = ...
    data: Any = ...
    files: Any = ...
    initial: Any = ...
    form_kwargs: Any = ...
    error_class: Any = ...
    def __init__(
        self,
        data: Optional[Any] = ...,
        files: Optional[Any] = ...,
        auto_id: str = ...,
        prefix: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        error_class: Any = ...,
        form_kwargs: Optional[Any] = ...,
    ) -> None: ...
    def __iter__(self): ...
    def __getitem__(self, index: Any): ...
    def __len__(self): ...
    def __bool__(self): ...
    def management_form(self): ...
    def total_form_count(self): ...
    def initial_form_count(self): ...
    @property
    def forms(self): ...
    def get_form_kwargs(self, index: Any): ...
    @property
    def initial_forms(self): ...
    @property
    def extra_forms(self): ...
    @property
    def empty_form(self): ...
    @property
    def cleaned_data(self): ...
    @property
    def deleted_forms(self): ...
    @property
    def ordered_forms(self): ...
    @classmethod
    def get_default_prefix(cls): ...
    def non_form_errors(self): ...
    @property
    def errors(self): ...
    def total_error_count(self): ...
    def is_valid(self): ...
    def full_clean(self): ...
    def clean(self) -> None: ...
    def has_changed(self): ...
    def add_fields(self, form: Any, index: Any) -> None: ...
    def add_prefix(self, index: Any): ...
    def is_multipart(self): ...
    @property
    def media(self): ...
    def as_table(self): ...
    def as_p(self): ...
    def as_ul(self): ...

def formset_factory(
    form: Any,
    formset: Any = ...,
    extra: int = ...,
    can_order: bool = ...,
    can_delete: bool = ...,
    max_num: Optional[Any] = ...,
    validate_max: bool = ...,
    min_num: Optional[Any] = ...,
    validate_min: bool = ...,
): ...
def all_valid(formsets: Sequence[Any]) -> bool: ...



from .core import (
    demojize as demojize,
    emoji_count as emoji_count,
    emoji_lis as emoji_lis,
    emojize as emojize,
    get_emoji_regexp as get_emoji_regexp,
)
from .unicode_codes import (
    EMOJI_ALIAS_UNICODE as EMOJI_ALIAS_UNICODE,
    EMOJI_UNICODE as EMOJI_UNICODE,
    UNICODE_EMOJI as UNICODE_EMOJI,
    UNICODE_EMOJI_ALIAS as UNICODE_EMOJI_ALIAS,
)



from socket import socket
from typing import Any, Mapping, Optional
from typing_extensions import Literal

from . import base_events, constants, events, futures, streams, transports

class _ProactorBasePipeTransport(transports._FlowControlMixin, transports.BaseTransport):
    def __init__(
        self,
        loop: events.AbstractEventLoop,
        sock: socket,
        protocol: streams.StreamReaderProtocol,
        waiter: Optional[futures.Future[Any]] = ...,
        extra: Optional[Mapping[Any, Any]] = ...,
        server: Optional[events.AbstractServer] = ...,
    ) -> None: ...
    def __repr__(self) -> str: ...
    def __del__(self) -> None: ...
    def get_write_buffer_size(self) -> int: ...

class _ProactorReadPipeTransport(_ProactorBasePipeTransport, transports.ReadTransport):
    def __init__(
        self,
        loop: events.AbstractEventLoop,
        sock: socket,
        protocol: streams.StreamReaderProtocol,
        waiter: Optional[futures.Future[Any]] = ...,
        extra: Optional[Mapping[Any, Any]] = ...,
        server: Optional[events.AbstractServer] = ...,
    ) -> None: ...

class _ProactorBaseWritePipeTransport(_ProactorBasePipeTransport, transports.WriteTransport):
    def __init__(
        self,
        loop: events.AbstractEventLoop,
        sock: socket,
        protocol: streams.StreamReaderProtocol,
        waiter: Optional[futures.Future[Any]] = ...,
        extra: Optional[Mapping[Any, Any]] = ...,
        server: Optional[events.AbstractServer] = ...,
    ) -> None: ...

class _ProactorWritePipeTransport(_ProactorBaseWritePipeTransport):
    def __init__(
        self,
        loop: events.AbstractEventLoop,
        sock: socket,
        protocol: streams.StreamReaderProtocol,
        waiter: Optional[futures.Future[Any]] = ...,
        extra: Optional[Mapping[Any, Any]] = ...,
        server: Optional[events.AbstractServer] = ...,
    ) -> None: ...

class _ProactorDuplexPipeTransport(_ProactorReadPipeTransport, _ProactorBaseWritePipeTransport, transports.Transport): ...

class _ProactorSocketTransport(_ProactorReadPipeTransport, _ProactorBaseWritePipeTransport, transports.Transport):

    _sendfile_compatible: constants._SendfileMode = ...
    def __init__(
        self,
        loop: events.AbstractEventLoop,
        sock: socket,
        protocol: streams.StreamReaderProtocol,
        waiter: Optional[futures.Future[Any]] = ...,
        extra: Optional[Mapping[Any, Any]] = ...,
        server: Optional[events.AbstractServer] = ...,
    ) -> None: ...
    def _set_extra(self, sock: socket) -> None: ...
    def can_write_eof(self) -> Literal[True]: ...
    def write_eof(self) -> None: ...

class BaseProactorEventLoop(base_events.BaseEventLoop):
    def __init__(self, proactor: Any) -> None: ...



import os
import sys
from types import CodeType, ModuleType
from typing import Any, List, Optional, Tuple, Union

if sys.version_info >= (3, 7):
    from importlib.abc import ResourceReader

class ZipImportError(ImportError): ...

class zipimporter(object):
    archive: str
    prefix: str
    if sys.version_info >= (3, 6):
        def __init__(self, path: Union[str, bytes, os.PathLike[Any]]) -> None: ...
    else:
        def __init__(self, path: Union[str, bytes]) -> None: ...
    if sys.version_info >= (3,):
        def find_loader(
            self, fullname: str, path: Optional[str] = ...
        ) -> Tuple[Optional[zipimporter], List[str]]: ...  # undocumented
    def find_module(self, fullname: str, path: Optional[str] = ...) -> Optional[zipimporter]: ...
    def get_code(self, fullname: str) -> CodeType: ...
    def get_data(self, pathname: str) -> str: ...
    def get_filename(self, fullname: str) -> str: ...
    if sys.version_info >= (3, 7):
        def get_resource_reader(self, fullname: str) -> Optional[ResourceReader]: ...  # undocumented
    def get_source(self, fullname: str) -> Optional[str]: ...
    def is_package(self, fullname: str) -> bool: ...
    def load_module(self, fullname: str) -> ModuleType: ...



import datetime
from typing import Any, Dict, Iterator, List, Optional, Text, Tuple, Type, TypeVar, Union

_RetType = Union[Type[float], Type[datetime.datetime]]
_SelfT = TypeVar("_SelfT", bound=croniter)

class CroniterError(ValueError): ...
class CroniterBadCronError(CroniterError): ...
class CroniterBadDateError(CroniterError): ...
class CroniterNotAlphaError(CroniterError): ...

class croniter(Iterator[Any]):
    MONTHS_IN_YEAR: int
    RANGES: Tuple[Tuple[int, int], ...]
    DAYS: Tuple[int, ...]
    ALPHACONV: Tuple[Dict[str, Any], ...]
    LOWMAP: Tuple[Dict[int, Any], ...]
    bad_length: str
    tzinfo: Optional[datetime.tzinfo]
    cur: float
    expanded: List[List[str]]
    start_time: float
    dst_start_time: float
    nth_weekday_of_month: Dict[str, Any]
    def __init__(
        self, expr_format: Text, start_time: Optional[Union[float, datetime.datetime]] = ..., ret_type: Optional[_RetType] = ...
    ) -> None: ...
    # Most return value depend on ret_type, which can be passed in both as a method argument and as
    # a constructor argument.
    def get_next(self, ret_type: Optional[_RetType] = ...) -> Any: ...
    def get_prev(self, ret_type: Optional[_RetType] = ...) -> Any: ...
    def get_current(self, ret_type: Optional[_RetType] = ...) -> Any: ...
    def __iter__(self: _SelfT) -> _SelfT: ...
    def __next__(self, ret_type: Optional[_RetType] = ...) -> Any: ...
    def next(self, ret_type: Optional[_RetType] = ...) -> Any: ...
    def all_next(self, ret_type: Optional[_RetType] = ...) -> Iterator[Any]: ...
    def all_prev(self, ret_type: Optional[_RetType] = ...) -> Iterator[Any]: ...
    def iter(self, ret_type: Optional[_RetType] = ...) -> Iterator[Any]: ...
    def is_leap(self, year: int) -> bool: ...
    @classmethod
    def expand(cls, expr_format: Text) -> Tuple[List[List[str]], Dict[str, Any]]: ...
    @classmethod
    def is_valid(cls, expression: Text) -> bool: ...



import ssl
from httplib import HTTPConnectionProtocol, HTTPResponse
from typing import Any, AnyStr, Callable, Dict, List, Mapping, Optional, Sequence, Text, Tuple, Type, Union
from urllib import addinfourl

_string = Union[str, unicode]

class URLError(IOError):
    reason: Union[str, BaseException]

class HTTPError(URLError, addinfourl):
    code: int
    headers: Mapping[str, str]
    def __init__(self, url, code: int, msg: str, hdrs: Mapping[str, str], fp: addinfourl) -> None: ...

class Request(object):
    host: str
    port: str
    data: str
    headers: Dict[str, str]
    unverifiable: bool
    type: Optional[str]
    origin_req_host = ...
    unredirected_hdrs: Dict[str, str]
    def __init__(
        self,
        url: str,
        data: Optional[str] = ...,
        headers: Dict[str, str] = ...,
        origin_req_host: Optional[str] = ...,
        unverifiable: bool = ...,
    ) -> None: ...
    def __getattr__(self, attr): ...
    def get_method(self) -> str: ...
    def add_data(self, data) -> None: ...
    def has_data(self) -> bool: ...
    def get_data(self) -> str: ...
    def get_full_url(self) -> str: ...
    def get_type(self): ...
    def get_host(self) -> str: ...
    def get_selector(self): ...
    def set_proxy(self, host, type) -> None: ...
    def has_proxy(self) -> bool: ...
    def get_origin_req_host(self) -> str: ...
    def is_unverifiable(self) -> bool: ...
    def add_header(self, key: str, val: str) -> None: ...
    def add_unredirected_header(self, key: str, val: str) -> None: ...
    def has_header(self, header_name: str) -> bool: ...
    def get_header(self, header_name: str, default: Optional[str] = ...) -> str: ...
    def header_items(self): ...

class OpenerDirector(object):
    addheaders: List[Tuple[str, str]]
    def add_handler(self, handler: BaseHandler) -> None: ...
    def open(
        self, fullurl: Union[Request, _string], data: Optional[_string] = ..., timeout: Optional[float] = ...
    ) -> Optional[addinfourl]: ...
    def error(self, proto: _string, *args: Any): ...

# Note that this type is somewhat a lie. The return *can* be None if
# a custom opener has been installed that fails to handle the request.
def urlopen(
    url: Union[Request, _string],
    data: Optional[_string] = ...,
    timeout: Optional[float] = ...,
    cafile: Optional[_string] = ...,
    capath: Optional[_string] = ...,
    cadefault: bool = ...,
    context: Optional[ssl.SSLContext] = ...,
) -> addinfourl: ...
def install_opener(opener: OpenerDirector) -> None: ...
def build_opener(*handlers: Union[BaseHandler, Type[BaseHandler]]) -> OpenerDirector: ...

class BaseHandler:
    handler_order: int
    parent: OpenerDirector
    def add_parent(self, parent: OpenerDirector) -> None: ...
    def close(self) -> None: ...
    def __lt__(self, other: Any) -> bool: ...

class HTTPErrorProcessor(BaseHandler):
    def http_response(self, request, response): ...

class HTTPDefaultErrorHandler(BaseHandler):
    def http_error_default(self, req: Request, fp: addinfourl, code: int, msg: str, hdrs: Mapping[str, str]): ...

class HTTPRedirectHandler(BaseHandler):
    max_repeats: int
    max_redirections: int
    def redirect_request(self, req: Request, fp: addinfourl, code: int, msg: str, headers: Mapping[str, str], newurl): ...
    def http_error_301(self, req: Request, fp: addinfourl, code: int, msg: str, headers: Mapping[str, str]): ...
    def http_error_302(self, req: Request, fp: addinfourl, code: int, msg: str, headers: Mapping[str, str]): ...
    def http_error_303(self, req: Request, fp: addinfourl, code: int, msg: str, headers: Mapping[str, str]): ...
    def http_error_307(self, req: Request, fp: addinfourl, code: int, msg: str, headers: Mapping[str, str]): ...
    inf_msg: str

class ProxyHandler(BaseHandler):
    proxies: Mapping[str, str]
    def __init__(self, proxies: Optional[Mapping[str, str]] = ...): ...
    def proxy_open(self, req: Request, proxy, type): ...

class HTTPPasswordMgr:
    def __init__(self) -> None: ...
    def add_password(self, realm: Optional[Text], uri: Union[Text, Sequence[Text]], user: Text, passwd: Text) -> None: ...
    def find_user_password(self, realm: Optional[Text], authuri: Text) -> Tuple[Any, Any]: ...
    def reduce_uri(self, uri: _string, default_port: bool = ...) -> Tuple[Any, Any]: ...
    def is_suburi(self, base: _string, test: _string) -> bool: ...

class HTTPPasswordMgrWithDefaultRealm(HTTPPasswordMgr): ...

class AbstractBasicAuthHandler:
    def __init__(self, password_mgr: Optional[HTTPPasswordMgr] = ...) -> None: ...
    def add_password(self, realm: Optional[Text], uri: Union[Text, Sequence[Text]], user: Text, passwd: Text) -> None: ...
    def http_error_auth_reqed(self, authreq, host, req: Request, headers: Mapping[str, str]): ...
    def retry_http_basic_auth(self, host, req: Request, realm): ...

class HTTPBasicAuthHandler(AbstractBasicAuthHandler, BaseHandler):
    auth_header: str
    def http_error_401(self, req: Request, fp: addinfourl, code: int, msg: str, headers: Mapping[str, str]): ...

class ProxyBasicAuthHandler(AbstractBasicAuthHandler, BaseHandler):
    auth_header: str
    def http_error_407(self, req: Request, fp: addinfourl, code: int, msg: str, headers: Mapping[str, str]): ...

class AbstractDigestAuthHandler:
    def __init__(self, passwd: Optional[HTTPPasswordMgr] = ...) -> None: ...
    def add_password(self, realm: Optional[Text], uri: Union[Text, Sequence[Text]], user: Text, passwd: Text) -> None: ...
    def reset_retry_count(self) -> None: ...
    def http_error_auth_reqed(self, auth_header: str, host: str, req: Request, headers: Mapping[str, str]) -> None: ...
    def retry_http_digest_auth(self, req: Request, auth: str) -> Optional[HTTPResponse]: ...
    def get_cnonce(self, nonce: str) -> str: ...
    def get_authorization(self, req: Request, chal: Mapping[str, str]) -> str: ...
    def get_algorithm_impls(self, algorithm: str) -> Tuple[Callable[[str], str], Callable[[str, str], str]]: ...
    def get_entity_digest(self, data: Optional[bytes], chal: Mapping[str, str]) -> Optional[str]: ...

class HTTPDigestAuthHandler(BaseHandler, AbstractDigestAuthHandler):
    auth_header: str
    handler_order: int
    def http_error_401(self, req: Request, fp: addinfourl, code: int, msg: str, headers: Mapping[str, str]): ...

class ProxyDigestAuthHandler(BaseHandler, AbstractDigestAuthHandler):
    auth_header: str
    handler_order: int
    def http_error_407(self, req: Request, fp: addinfourl, code: int, msg: str, headers: Mapping[str, str]): ...

class AbstractHTTPHandler(BaseHandler):  # undocumented
    def __init__(self, debuglevel: int = ...) -> None: ...
    def set_http_debuglevel(self, level: int) -> None: ...
    def do_request_(self, request: Request) -> Request: ...
    def do_open(self, http_class: HTTPConnectionProtocol, req: Request, **http_conn_args: Optional[Any]) -> addinfourl: ...

class HTTPHandler(AbstractHTTPHandler):
    def http_open(self, req: Request) -> addinfourl: ...
    def http_request(self, request: Request) -> Request: ...  # undocumented

class HTTPSHandler(AbstractHTTPHandler):
    def __init__(self, debuglevel: int = ..., context: Optional[ssl.SSLContext] = ...) -> None: ...
    def https_open(self, req: Request) -> addinfourl: ...
    def https_request(self, request: Request) -> Request: ...  # undocumented

class HTTPCookieProcessor(BaseHandler):
    def __init__(self, cookiejar: Optional[Any] = ...): ...
    def http_request(self, request: Request): ...
    def http_response(self, request: Request, response): ...

class UnknownHandler(BaseHandler):
    def unknown_open(self, req: Request): ...

class FileHandler(BaseHandler):
    def file_open(self, req: Request): ...
    def get_names(self): ...
    def open_local_file(self, req: Request): ...

class FTPHandler(BaseHandler):
    def ftp_open(self, req: Request): ...
    def connect_ftp(self, user, passwd, host, port, dirs, timeout): ...

class CacheFTPHandler(FTPHandler):
    def __init__(self) -> None: ...
    def setTimeout(self, t: Optional[float]): ...
    def setMaxConns(self, m: int): ...
    def check_cache(self): ...
    def clear_cache(self): ...

def parse_http_list(s: AnyStr) -> List[AnyStr]: ...
def parse_keqv_list(l: List[AnyStr]) -> Dict[AnyStr, AnyStr]: ...



from typing import Any

from . import fields

RequestField = fields.RequestField

writer: Any

def choose_boundary(): ...
def iter_field_objects(fields): ...
def iter_fields(fields): ...
def encode_multipart_formdata(fields, boundary=...): ...



from typing import Iterator, Optional, Sequence, Tuple

from paramiko.file import BufferedFile
from paramiko.sftp_attr import SFTPAttributes
from paramiko.sftp_client import SFTPClient
from paramiko.sftp_handle import SFTPHandle

class SFTPFile(BufferedFile):
    MAX_REQUEST_SIZE: int
    sftp: SFTPClient
    handle: SFTPHandle
    pipelined: bool
    def __init__(self, sftp: SFTPClient, handle: bytes, mode: str = ..., bufsize: int = ...) -> None: ...
    def __del__(self) -> None: ...
    def close(self) -> None: ...
    def settimeout(self, timeout: float) -> None: ...
    def gettimeout(self) -> float: ...
    def setblocking(self, blocking: bool) -> None: ...
    def seekable(self) -> bool: ...
    def seek(self, offset: int, whence: int = ...) -> None: ...
    def stat(self) -> SFTPAttributes: ...
    def chmod(self, mode: int) -> None: ...
    def chown(self, uid: int, gid: int) -> None: ...
    def utime(self, times: Optional[Tuple[float, float]]) -> None: ...
    def truncate(self, size: int) -> None: ...
    def check(self, hash_algorithm: str, offset: int = ..., length: int = ..., block_size: int = ...) -> bytes: ...
    def set_pipelined(self, pipelined: bool = ...) -> None: ...
    def prefetch(self, file_size: Optional[int] = ...) -> None: ...
    def readv(self, chunks: Sequence[Tuple[int, int]]) -> Iterator[bytes]: ...



from typing import Any, Pattern

from markdown.blockprocessors import BlockProcessor
from markdown.extensions import Extension

class AdmonitionExtension(Extension): ...

class AdmonitionProcessor(BlockProcessor):
    CLASSNAME: str = ...
    CLASSNAME_TITLE: str = ...
    RE: Pattern
    RE_SPACES: Any
    def get_class_and_title(self, match): ...

def makeExtension(**kwargs): ...



from typing import Any, Dict, Optional

from werkzeug.exceptions import HTTPException
from werkzeug.routing import Rule
from werkzeug.wrappers import Request as RequestBase, Response as ResponseBase

class JSONMixin:
    @property
    def is_json(self) -> bool: ...
    @property
    def json(self): ...
    def get_json(self, force: bool = ..., silent: bool = ..., cache: bool = ...): ...
    def on_json_loading_failed(self, e: Any) -> None: ...

class Request(RequestBase, JSONMixin):
    url_rule: Optional[Rule] = ...
    view_args: Dict[str, Any] = ...
    routing_exception: Optional[HTTPException] = ...
    # Request is making the max_content_length readonly, where it was not the
    # case in its supertype.
    # We would require something like https://github.com/python/typing/issues/241
    @property
    def max_content_length(self) -> Optional[int]: ...  # type: ignore
    @property
    def endpoint(self) -> Optional[str]: ...
    @property
    def blueprint(self) -> Optional[str]: ...

class Response(ResponseBase, JSONMixin):
    default_mimetype: Optional[str] = ...
    @property
    def max_cookie_size(self) -> int: ...



from collections import OrderedDict
from typing import Iterator, List, Optional, Tuple, Union

from django.core.files.storage import FileSystemStorage

def matches_patterns(path: str, patterns: Union[List[str], Tuple[str], OrderedDict] = ...) -> bool: ...
def get_files(storage: FileSystemStorage, ignore_patterns: List[str] = ..., location: str = ...) -> Iterator[str]: ...
def check_settings(base_url: Optional[str] = ...) -> None: ...



from types import ModuleType
from typing import Any

from werkzeug import (
    _internal,
    datastructures,
    debug,
    exceptions,
    formparser,
    http,
    local,
    security,
    serving,
    test,
    testapp,
    urls,
    useragents,
    utils,
    wrappers,
    wsgi,
)

class module(ModuleType):
    def __getattr__(self, name): ...
    def __dir__(self): ...

__version__: Any

run_simple = serving.run_simple
test_app = testapp.test_app
UserAgent = useragents.UserAgent
_easteregg = _internal._easteregg
DebuggedApplication = debug.DebuggedApplication
MultiDict = datastructures.MultiDict
CombinedMultiDict = datastructures.CombinedMultiDict
Headers = datastructures.Headers
EnvironHeaders = datastructures.EnvironHeaders
ImmutableList = datastructures.ImmutableList
ImmutableDict = datastructures.ImmutableDict
ImmutableMultiDict = datastructures.ImmutableMultiDict
TypeConversionDict = datastructures.TypeConversionDict
ImmutableTypeConversionDict = datastructures.ImmutableTypeConversionDict
Accept = datastructures.Accept
MIMEAccept = datastructures.MIMEAccept
CharsetAccept = datastructures.CharsetAccept
LanguageAccept = datastructures.LanguageAccept
RequestCacheControl = datastructures.RequestCacheControl
ResponseCacheControl = datastructures.ResponseCacheControl
ETags = datastructures.ETags
HeaderSet = datastructures.HeaderSet
WWWAuthenticate = datastructures.WWWAuthenticate
Authorization = datastructures.Authorization
FileMultiDict = datastructures.FileMultiDict
CallbackDict = datastructures.CallbackDict
FileStorage = datastructures.FileStorage
OrderedMultiDict = datastructures.OrderedMultiDict
ImmutableOrderedMultiDict = datastructures.ImmutableOrderedMultiDict
escape = utils.escape
environ_property = utils.environ_property
append_slash_redirect = utils.append_slash_redirect
redirect = utils.redirect
cached_property = utils.cached_property
import_string = utils.import_string
dump_cookie = http.dump_cookie
parse_cookie = http.parse_cookie
unescape = utils.unescape
format_string = utils.format_string
find_modules = utils.find_modules
header_property = utils.header_property
html = utils.html
xhtml = utils.xhtml
HTMLBuilder = utils.HTMLBuilder
validate_arguments = utils.validate_arguments
ArgumentValidationError = utils.ArgumentValidationError
bind_arguments = utils.bind_arguments
secure_filename = utils.secure_filename
BaseResponse = wrappers.BaseResponse
BaseRequest = wrappers.BaseRequest
Request = wrappers.Request
Response = wrappers.Response
AcceptMixin = wrappers.AcceptMixin
ETagRequestMixin = wrappers.ETagRequestMixin
ETagResponseMixin = wrappers.ETagResponseMixin
ResponseStreamMixin = wrappers.ResponseStreamMixin
CommonResponseDescriptorsMixin = wrappers.CommonResponseDescriptorsMixin
UserAgentMixin = wrappers.UserAgentMixin
AuthorizationMixin = wrappers.AuthorizationMixin
WWWAuthenticateMixin = wrappers.WWWAuthenticateMixin
CommonRequestDescriptorsMixin = wrappers.CommonRequestDescriptorsMixin
Local = local.Local
LocalManager = local.LocalManager
LocalProxy = local.LocalProxy
LocalStack = local.LocalStack
release_local = local.release_local
generate_password_hash = security.generate_password_hash
check_password_hash = security.check_password_hash
Client = test.Client
EnvironBuilder = test.EnvironBuilder
create_environ = test.create_environ
run_wsgi_app = test.run_wsgi_app
get_current_url = wsgi.get_current_url
get_host = wsgi.get_host
pop_path_info = wsgi.pop_path_info
peek_path_info = wsgi.peek_path_info
SharedDataMiddleware = wsgi.SharedDataMiddleware
DispatcherMiddleware = wsgi.DispatcherMiddleware
ClosingIterator = wsgi.ClosingIterator
FileWrapper = wsgi.FileWrapper
make_line_iter = wsgi.make_line_iter
LimitedStream = wsgi.LimitedStream
responder = wsgi.responder
wrap_file = wsgi.wrap_file
extract_path_info = wsgi.extract_path_info
parse_etags = http.parse_etags
parse_date = http.parse_date
http_date = http.http_date
cookie_date = http.cookie_date
parse_cache_control_header = http.parse_cache_control_header
is_resource_modified = http.is_resource_modified
parse_accept_header = http.parse_accept_header
parse_set_header = http.parse_set_header
quote_etag = http.quote_etag
unquote_etag = http.unquote_etag
generate_etag = http.generate_etag
dump_header = http.dump_header
parse_list_header = http.parse_list_header
parse_dict_header = http.parse_dict_header
parse_authorization_header = http.parse_authorization_header
parse_www_authenticate_header = http.parse_www_authenticate_header
remove_entity_headers = http.remove_entity_headers
is_entity_header = http.is_entity_header
remove_hop_by_hop_headers = http.remove_hop_by_hop_headers
parse_options_header = http.parse_options_header
dump_options_header = http.dump_options_header
is_hop_by_hop_header = http.is_hop_by_hop_header
unquote_header_value = http.unquote_header_value
quote_header_value = http.quote_header_value
HTTP_STATUS_CODES = http.HTTP_STATUS_CODES
url_decode = urls.url_decode
url_encode = urls.url_encode
url_quote = urls.url_quote
url_quote_plus = urls.url_quote_plus
url_unquote = urls.url_unquote
url_unquote_plus = urls.url_unquote_plus
url_fix = urls.url_fix
Href = urls.Href
iri_to_uri = urls.iri_to_uri
uri_to_iri = urls.uri_to_iri
parse_form_data = formparser.parse_form_data
abort = exceptions.Aborter
Aborter = exceptions.Aborter



from typing import Any, List, Optional, Type

from pyVmomi.vim import ManagedEntity
from pyVmomi.vim.view import ContainerView
from pyVmomi.vmodl import DynamicProperty

class PropertyCollector:
    class PropertySpec:
        all: bool
        type: Type[ManagedEntity]
        pathSet: List[str]
    class TraversalSpec:
        path: str
        skip: bool
        type: Type[ContainerView]
        def __getattr__(self, name: str) -> Any: ...  # incomplete
    class RetrieveOptions:
        maxObjects: int
    class ObjectSpec:
        skip: bool
        selectSet: List[PropertyCollector.TraversalSpec]
        obj: Any
    class FilterSpec:
        propSet: List[PropertyCollector.PropertySpec]
        objectSet: List[PropertyCollector.ObjectSpec]
        def __getattr__(self, name: str) -> Any: ...  # incomplete
    class ObjectContent:
        obj: ManagedEntity
        propSet: List[DynamicProperty]
        def __getattr__(self, name: str) -> Any: ...  # incomplete
    class RetrieveResult:
        objects: List[PropertyCollector.ObjectContent]
        token: Optional[str]
    def RetrievePropertiesEx(
        self, specSet: List[PropertyCollector.FilterSpec], options: PropertyCollector.RetrieveOptions
    ) -> PropertyCollector.RetrieveResult: ...
    def ContinueRetrievePropertiesEx(self, token: str) -> PropertyCollector.RetrieveResult: ...
    def __getattr__(self, name: str) -> Any: ...  # incomplete



from typing import Any, Dict, List

from django.db.models.deletion import Collector

from django.core.management import BaseCommand

class Command(BaseCommand): ...

class NoFastDeleteCollector(Collector):
    data: Dict[str, Any]
    dependencies: Dict[Any, Any]
    fast_deletes: List[Any]
    field_updates: Dict[Any, Any]
    using: str
    def can_fast_delete(self, *args: Any, **kwargs: Any) -> bool: ...



import sys
from typing import Any, Awaitable, Callable, Iterable, List, Optional, Tuple

from . import events

if sys.version_info >= (3, 8):
    async def staggered_race(
        coro_fns: Iterable[Callable[[], Awaitable[Any]]],
        delay: Optional[float],
        *,
        loop: Optional[events.AbstractEventLoop] = ...,
    ) -> Tuple[Any, Optional[int], List[Optional[Exception]]]: ...



class _JediError(Exception):
    pass


class InternalError(_JediError):
    """
    This error might happen a subprocess is crashing. The reason for this is
    usually broken C code in third party libraries. This is not a very common
    thing and it is safe to use Jedi again. However using the same calls might
    result in the same error again.
    """


class WrongVersion(_JediError):
    """
    This error is reserved for the future, shouldn't really be happening at the
    moment.
    """


class RefactoringError(_JediError):
    """
    Refactorings can fail for various reasons. So if you work with refactorings
    like :meth:`.Script.rename`, :meth:`.Script.inline`,
    :meth:`.Script.extract_variable` and :meth:`.Script.extract_function`, make
    sure to catch these. The descriptions in the errors are ususally valuable
    for end users.

    A typical ``RefactoringError`` would tell the user that inlining is not
    possible if no name is under the cursor.
    """
